from typing import NewType, Protocol, TypeVar

__doc__: str
T_co = TypeVar("T_co", covariant=True, bound=int)
Uint32 = NewType("Uint32", int)
Uint64 = NewType("Uint64", int)
Uint128 = NewType("Uint128", int)

class Hasher(Protocol[T_co]):
    def __init__(self, *, seed: int) -> None:
        """
        Summary
        -------
        Initialise `Hasher` with a a signed 64-bit `seed`.
        The `seed` should not be exposed as it is used to deterministically generate the hash.
        An exposed `seed` would put your service at a higher risk of a DoS attack.

        Parameters
        ----------
        seed (`int`)
            a signed 64-bit seed for the hasher [-2^63, 2^63)

        Example
        -------
        ```python
        >>> from gxhash import GxHash32
        >>> GxHash32(seed=2**63 - 1)
        <builtins.GxHash32 object at 0x...>

        ```
        """

    def hash(self, bytes: bytes, /) -> T_co:
        """
        Summary
        -------
        Hashes `bytes` to an `int`.
        This method has less overhead than `hash_async`.

        Parameters
        ----------
        bytes (`bytes`)
            input bytes

        Returns
        -------
        hash (`int`)
            the hash of the input bytes

        Example
        -------
        ```python
        >>> from gxhash import GxHash64
        >>> hasher = GxHash64(seed=1234)
        >>> hasher.hash(bytes(range(256)))
        12522596144082598891

        ```
        """

    async def hash_async(self, bytes: bytes, /) -> T_co:
        """
        Summary
        -------
        Hashes `bytes` to an `int` asynchronously.
        This method allows you to compute multiple hashes with true multi-threaded parallelism.
        If called sequentially, this method is slightly less performant than the default `hash` method.
        Otherwise, this variant offers the highest throughput.

        Parameters
        ----------
        bytes (`bytes`)
            input bytes

        Returns
        -------
        hash (`int`)
            the hash of the input bytes

        Example
        -------
        ```python
        >>> from gxhash import GxHash128
        >>> from asyncio import run
        >>> hasher = GxHash128(seed=1234)
        >>> run(hasher.hash_async(bytes(range(256))))
        117181821629540739333037011138327886827

        ```
        """

class GxHashAsyncError(Exception):
    """
    Summary
    -------
    This error is raised when an asynchronous hash operation fails.
    """

class GxHash32(Hasher[Uint32]):
    """
    Summary
    -------
    This class exposes GxHash's 32-bit hash methods.
    """

class GxHash64(Hasher[Uint64]):
    """
    Summary
    -------
    This class exposes GxHash's 64-bit hash methods.
    """

class GxHash128(Hasher[Uint128]):
    """
    Summary
    -------
    This class exposes GxHash's 128-bit hash methods.
    """



from _hashlib import HASH
from collections.abc import Callable
from collections.abc import Set as AbstractSet
from typing import Literal, Protocol

from gxhash.buffer import Buffer as Buffer

algorithms_available: AbstractSet[str]
algorithms_guaranteed: AbstractSet[str]

class FileLike(Protocol):
    def fileno(self) -> int: ...

class BytesIOLike(Protocol):
    def getbuffer(self) -> Buffer: ...

def new(
    name: Literal["gxhash32", "gxhash64", "gxhash128"],
    data: Buffer = b"",
    *,
    seed: int = 0,
    usedforsecurity: bool = False,
    **kwargs: object,
) -> HASH:
    """
    Summary
    -------
    Returns a hash object implementing the given hash algorithm; optionally initialised with `data`.
    Note that GxHash is not an incremental hasher,
    and all inputs provided to the `update` method are accumulated internally.

    Parameters
    ----------
    name (`str`)
        the name of the hash algorithm to use; must be one of "gxhash32", "gxhash64", or "gxhash128"

    data (`Buffer?`)
        input data to initialise the hasher

    seed (`int?`)
        a signed 64-bit seed for the hasher [-2^63, 2^63)

    usedforsecurity (`bool?`)
        this parameter has no effect and is only present for compatibility with `hashlib`

    Returns
    -------
    gxhash (`HASH`)
        the GxHash object

    Example
    -------
    ```python
    >>> from gxhash.hashlib import new
    >>> hasher = new("gxhash32", b"hello", seed=42)
    >>> hasher.hexdigest()
    '9ffaa800'

    ```
    """

def file_digest(
    fileobj: BytesIOLike | FileLike,
    digest: str | Callable[[], HASH],
    *,
    seed: int = 0,
    **kwargs: object,
) -> HASH:
    """
    Summary
    -------
    Returns a hash object implementing the given hash algorithm, with the hash of the file-like object.

    Parameters
    ----------
    fileobj (`BytesIOLike | FileLike`)
        a file-like object with a fileno() method

    digest (`str | Callable[[], HASH]`)
        the name of the hash algorithm to use, or a zero-argument callable that returns a new hash object

    seed (`int?`)
        a signed 64-bit seed for the hasher [-2^63, 2^63); only used if `digest` is a string

    Returns
    -------
    gxhash (`HASH`)
        the hash object with the file's hash

    Example
    -------
    ```python
    >>> from gxhash.hashlib import file_digest
    >>> from io import BytesIO
    >>> file = BytesIO(b"hello")
    >>> hasher = file_digest(file, "gxhash32", seed=42)
    >>> hasher.digest()
    b'\x9f\xfa\xa8\x00'

    ```
    """

def gxhash32(data: Buffer = b"", *, seed: int = 0, usedforsecurity: bool = False, **kwargs: object) -> HASH:
    """
    Summary
    -------
    Returns a GxHash32 hash object; optionally initialised with `data`.
    Note that GxHash is not an incremental hasher,
    and all inputs provided to the `update` method are accumulated internally.

    Parameters
    ----------
    data (`Buffer?`)
        input data to initialise the hasher

    seed (`int?`)
        a signed 64-bit seed for the hasher [-2^63, 2^63)

    usedforsecurity (`bool?`)
        this parameter has no effect and is only present for compatibility with `hashlib`

    Returns
    -------
    gxhash (`HASH`)
        the GxHash32 hash object

    Example
    -------
    ```python
    >>> from gxhash.hashlib import gxhash32
    >>> hasher = gxhash32(b"hello", seed=1234)
    >>> hasher.hexdigest()
    '72d987dc'

    ```
    """

def gxhash64(data: Buffer = b"", *, seed: int = 0, usedforsecurity: bool = False, **kwargs: object) -> HASH:
    """
    Summary
    -------
    Returns a GxHash64 hash object; optionally initialised with `data`.
    Note that GxHash is not an incremental hasher,
    and all inputs provided to the `update` method are accumulated internally.

    Parameters
    ----------
    data (`Buffer?`)
        input data to initialise the hasher

    seed (`int?`)
        a signed 64-bit seed for the hasher [-2^63, 2^63)

    usedforsecurity (`bool?`)
        this parameter has no effect and is only present for compatibility with `hashlib`

    Returns
    -------
    gxhash (`HASH`)
        the GxHash64 hash object

    Example
    -------
    ```python
    >>> from gxhash.hashlib import gxhash64
    >>> hasher = gxhash64(b"hello", seed=1234)
    >>> hasher.hexdigest()
    '72d987dc0ecdfa46'

    ```
    """

def gxhash128(data: Buffer = b"", *, seed: int = 0, usedforsecurity: bool = False, **kwargs: object) -> HASH:
    """
    Summary
    -------
    Returns a GxHash128 hash object; optionally initialised with `data`.
    Note that GxHash is not an incremental hasher,
    and all inputs provided to the `update` method are accumulated internally.

    Parameters
    ----------
    data (`Buffer?`)
        input data to initialise the hasher

    seed (`int?`)
        a signed 64-bit seed for the hasher [-2^63, 2^63)

    usedforsecurity (`bool?`)
        this parameter has no effect and is only present for compatibility with `hashlib`

    Returns
    -------
    gxhash (`HASH`)
        the GxHash128 hash object

    Example
    -------
    ```python
    >>> from gxhash.hashlib import gxhash128
    >>> hasher = gxhash128(b"hello", seed=1234)
    >>> hasher.hexdigest()
    '72d987dc0ecdfa46a11353e202d601bc'

    ```
    """



from gxhash.gxhashlib import __doc__ as __doc__
from gxhash.gxhashlib import algorithms_available as algorithms_available
from gxhash.gxhashlib import algorithms_guaranteed as algorithms_guaranteed
from gxhash.gxhashlib import file_digest as file_digest
from gxhash.gxhashlib import gxhash32 as gxhash32
from gxhash.gxhashlib import gxhash64 as gxhash64
from gxhash.gxhashlib import gxhash128 as gxhash128
from gxhash.gxhashlib import new as new



from . import gxhash as gxhash  # noqa: I001

from gxhash.core import GxHash32 as GxHash32
from gxhash.core import GxHash64 as GxHash64
from gxhash.core import GxHash128 as GxHash128
from gxhash.core import GxHashAsyncError as GxHashAsyncError
from gxhash.core import __doc__ as __doc__
